﻿using System;
using System.Windows.Forms;

using Mcs.CMOS.StreamIO;

namespace Mcs.CMOS.DataClient
{
    public partial class UIDataClient : UserControl
    {
        #region events

        public enum enCommand
        {
            ConnectDisconnect
        }
        
        public class ControlEventArgs : System.EventArgs
        {
            public enCommand Command { get; private set; }
            public string Message { get; private set; }
            public ControlEventArgs(enCommand command, string message = "")
            {
                Command = command;
                Message = message;
            }
        }
        public event Action<ControlEventArgs> EvtControl;

        #endregion


        public UIDataClient()
        {
            InitializeComponent();
        }

        private delegate void DelegateShowLog(string msg);
        void ShowLog(string msg)
        {
            if (InvokeRequired)
            {
                Invoke(new DelegateShowLog(ShowLog), new object[] { msg });
            }
            else
            {
                textBoxLog.AppendText(" < " + DateTime.Now.ToString("T") + " >  " + msg + "\r\n");
            }
        }

        private delegate void DelegateSetClientState(PipeClient.enState state, string message);
        public void SetClientState(PipeClient.enState state, string message)
        {
            if (InvokeRequired)
            {
                Invoke(new DelegateSetClientState(SetClientState), new object[] { state, message });
            }
            else
            {
                textBoxLog.AppendText(" < " + DateTime.Now.ToString("T") + " >  " + message + "\r\n"); ;

                if (state == PipeClient.enState.Connected)
                {
                    toolStripButtonConnectDisconnect.Image = Mcs.CMOS.DataClient.Properties.Resources.CMOS_MEA_Control_stop_24;
                }
                else if (state == PipeClient.enState.Disconnected)
                {
                    toolStripButtonConnectDisconnect.Image = Mcs.CMOS.DataClient.Properties.Resources.CMOS_MEA_Control_play_24;
                }
            }
        }

        private delegate void DelegateLogClientEvent(PipeClient.enEvent state, string message);
        public void LogClientEvent(PipeClient.enEvent state, string message)
        {
            if (InvokeRequired)
            {
                Invoke(new DelegateLogClientEvent(LogClientEvent), new object[] { state, message });
            }
            else
            {
                textBoxLog.AppendText(" < " + DateTime.Now.ToString("T") + " >  " + message + "\r\n");
            }
        }

        private void OnToolStripButtonConnectDisconnect(object sender, EventArgs e)
        {
            SendCommand(enCommand.ConnectDisconnect, toolStripTextBoxComputerName.Text);
        }


        private void SendCommand(enCommand command, string message)
        {
            if (EvtControl != null)
            {
                EvtControl(new ControlEventArgs(command, message));
            }
        }

        private void OnToolStripButtonSettings(object sender, EventArgs e)
        {

        }
    }
}
