﻿using System;
using System.Windows.Forms;
using System.Collections.Generic;

using Mcs.CMOS.Base;
using Mcs.DataStream;
using Mcs.CMOS.GuiTools;
using Mcs.TChart.FastLine;
using Mcs.MeaTrace;



namespace Mcs.CMOS.SpikeView
{
    public partial class SpikeExplorerView : UserControl//, ICMosToolView
    {
        private enState state = enState.Disabled;
        //private Data.enMode mode = Data.enMode.Segment;

        public SensorArrayView SensorArray { get; set; }
        public FastLine RasterPlot { get; set; }
        public MeaTraces Traces { get; private set; }

        public enum enState
        {
            Idle,
            ProcessData,
            ExtractUnits,
            Disabled
        }

        public enum enCommand
        {
            StartProcess,
            StopProcess,
            Export,
            Settings
        }

        public class ControlEventArgs : System.EventArgs
        {
            public enCommand Command { get; private set; }
            public ControlEventArgs(enCommand command)
            {
                Command = command;
            }
        }
        public event Action<ControlEventArgs> EvtControl;


        public bool ToolStripVisible
        {
            get
            {
                return toolStrip.Visible;
            }
            set
            {
                toolStrip.Visible = value;
            }
        }


        public SpikeExplorerView()
        {
            InitializeComponent();

            //WaveformPlot = new Mcs.CMOS.SpikeView.WaveformPlot();
            //WaveformPlot.Dock = System.Windows.Forms.DockStyle.Fill;
            //WaveformPlot.Location = new System.Drawing.Point(0, 0);
            //WaveformPlot.Name = "waveformPlot";
            //WaveformPlot.BackColor = System.Drawing.Color.Red;
            //splitContainerLeftRight.Panel2.Controls.Add(WaveformPlot);
            Traces = new MeaTrace.MeaTraces();
            Traces.AutoLayout = false;
            Traces.ToolStripVisible = true;
            Traces.ToolStripYAxisControlsVisible = true;
            Traces.ToolStripXSpanControlVisible = true;
            Traces.ToolStripExcludeControlVisible = false;
            Traces.ToolStripSelectControlVisible = false;
            Traces.ToolStripSelectDisplayModeVisible = false;
            Traces.ToolStripOverlayControlsVisible = false;
            Traces.SetDisplayMode(MeaTraceSettings.enDisplayMode.Overlay);
            splitContainerLeftRight.Panel2.Controls.Add(Traces);

            RasterPlot = new FastLine(-1, FastLine.enStyle.Raster);
            RasterPlot.Dock = DockStyle.Fill;
            RasterPlot.ToolStripVisible = false;
            RasterPlot.AutoAdjustYAxes = false;
            RasterPlot.XAxisVisible = true;
            RasterPlot.YAxisVisible = true;
            RasterPlot.LegendVisible = false;
            RasterPlot.Settings.SignalTimeUnit = new Unit(enUnit.s, -6);
            RasterPlot.Settings.XAxesUnit = new Unit(enUnit.s, 0);
            RasterPlot.Settings.YAxesUnit = new Unit(enUnit.NoUnit, 0);
            RasterPlot.Settings.YAxesMin = 0;
            RasterPlot.Settings.YAxesMax = 10;
            ((SettingsRasterTrace)RasterPlot.Settings).ShowTraceLabels = true;

            splitContainerTopBottom.Panel2.Controls.Add(RasterPlot);

            SensorArray = new SensorArrayView();
            SensorArray.ToolStripVisible = true;
            SensorArray.ToolStripSelectSensorsVisible = true;
            //SensorArray.ToolStripSelectSensorsSettingsVisible = false;
            SensorArray.ToolStripRangeVisible = true;
            SensorArray.ToolStripOpenSettingsDialogVisible = true;

            SensorArray.EnableRoiResize = false;
            SensorArray.RoiSelectionMode = MeaField.enSelectionMode.SingleSensors;
            SensorArray.SettingsRois.DefaultRoiCursor.ShowCrossHairLines = false;
            splitContainerLeftRight.Panel1.Controls.Add(SensorArray);
        }


        //private delegate void DelegateSetMode(Data.enMode mode);
        //public void SetMode(Data.enMode mode)
        //{
        //    if (InvokeRequired)
        //    {
        //        Invoke(new DelegateSetMode(SetMode), new object[] { mode });
        //    }
        //    else
        //    {
        //        this.mode = mode;

        //        if (state == enState.ProcessData || (state == enState.Idle && mode == Data.enMode.Segment))
        //        {
        //            toolStripButtonStartStopProcess.Enabled = true;
        //        }
        //        else
        //        {
        //            toolStripButtonStartStopProcess.Enabled = false;
        //        }
        //    }
        //}


        private delegate void DelegateSetState(enState state);
        public void SetState(enState state)
        {
            if (InvokeRequired)
            {
                Invoke(new DelegateSetState(SetState), new object[] { state });
            }
            else
            {
                this.state = state;
                if (state == enState.ProcessData)
                {
                    toolStripButtonSettings.Enabled = false;
                    toolStripButtonExport.Enabled = false;
                    toolStripButtonStartStopProcess.Enabled = true;
                    toolStripButtonStartStopProcess.Image = Mcs.CMOS.SpikeView.Properties.Resources.CMOS_MEA_Control_stop_24;
                }
                else if (state == enState.Idle)
                {
                    toolStripButtonSettings.Enabled = true;
                    toolStripButtonExport.Enabled = true;
                    toolStripButtonStartStopProcess.Enabled = true; //mode == Data.enMode.Segment;
                    toolStripButtonStartStopProcess.Image = Mcs.CMOS.SpikeView.Properties.Resources.CMOS_MEA_Control_process;
                }
                else if (state == enState.Disabled)
                {
                    toolStripButtonSettings.Enabled = false;
                    toolStripButtonExport.Enabled = false;
                    toolStripButtonStartStopProcess.Enabled = false;
                }
            }
        }

        #region UI Interaction

        private void SendCommand(enCommand command)
        {
            if (EvtControl != null)
            {
                EvtControl(new ControlEventArgs(command));
            }
        }

        private void OnToolStripButtonStartStopProcessClick(object sender, EventArgs e)
        {
            if (state == enState.Idle)
            {
                SendCommand(enCommand.StartProcess);
            }
            else if (state == enState.ProcessData)
            {
                SendCommand(enCommand.StopProcess);
            }
        }

        private void OnToolStripButtonExportClick(object sender, EventArgs e)
        {
            SendCommand(enCommand.Export);
        }

        private void OnToolStripButtonSettingsClick(object sender, EventArgs e)
        {
            SendCommand(enCommand.Settings);
        }

        #endregion
    }
}
